from rallf.tools.communicator import Communicator


class Logger(Communicator):
    """
        Log severity codes according with RFC 3164
        @see https://www.ietf.org/rfc/rfc3164.txt
    """

    def __init__(self, input, output):
        super().__init__(input, output)

    def log(self, msg, severity=7, data=None):
        return self.rpccall("log", {"message": str(msg), "severity": severity, "data": data}, wait=False)

    def debug(self, msg, data=None):
        """Debug: debug-level messages"""
        return self.log(msg, 7, data)

    def info(self, msg, data=None):
        """Informational: informational messages"""
        return self.log(msg, 6, data)

    def notice(self, msg, data=None):
        """Notice: normal but significant condition"""
        return self.log(msg, 5, data)

    def warning(self, msg, data=None):
        """Warning: warning conditions"""
        return self.log(msg, 4, data)

    def error(self, msg, data=None):
        """Error: error conditions"""
        return self.log(msg, 3, data)

    def critical(self, msg, data=None):
        """Critical: critical conditions"""
        return self.log(msg, 2, data)

    def alert(self, msg, data=None):
        """Alert: action must be taken immediately"""
        return self.log(msg, 1, data)

    def emergency(self, msg, data=None):
        """Emergency: system is unusable"""
        return self.log(msg, 0, data)



from rallf.tools.communicator import Communicator


class Network(Communicator):
    def __init__(self, input, output):
        super().__init__(input, output)

    def call(self, kind, routine, args):
        return self.rpccall(kind, {"routine": routine, "params": args})

    def use(self, routine, args=None):
        return self.call("delegate_local", routine, args)

    def delegate(self, routine, args=None):
        return self.call("delegate_remote", routine, args)

    def event(self, name, data=None):
        return self.rpccall("event", {"name": name, "data": data}, wait=False)



from json.decoder import JSONDecodeError

from rallf.sdk.rallf_error import RallfError
from rallf.tools.communicator import Communicator


class Listener(Communicator):
    def __init__(self, input, output):
        super().__init__(input, output)

    def listen(self, task):
        try:
            req = self.wait()
            if 'method' in req:
                try:
                    resp = getattr(task, req['params']['routine'])(req['params']['args'])
                    task.network.rpcresponse(id=req['id'], result=resp, error=False)
                except RallfError as e:
                    task.network.rpcresponse(id=req['id'], error=e.dict(), result=False)
            else:
                self.mq.append(req)

        except JSONDecodeError:
            task.network.rpcresponse(id=None, error={"code": -32700, "message": "Parse error"})
        except (EOFError, InterruptedError):
            task.finish()



import os

from selenium.webdriver import FirefoxProfile
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
from selenium.webdriver.firefox.webdriver import WebDriver as FirefoxDriver
from appium import webdriver as appium_webdriver

from rallf.sdk.rallf_error import RallfError


class SeleniumDeviceFactory:
    def __init__(self, robot):
        self.robot = robot

    def build(self, key, profile_dir=None):
        definition = self.robot.devices[key]
        if 'bin' in definition and not os.path.isabs(definition['bin']):
            definition['bin'] = os.path.abspath("%s/bin/%s" % (self.robot.home, definition['bin']))
        if 'driver' in definition and not os.path.isabs(definition['driver']):
            definition['driver'] = os.path.abspath("%s/bin/%s" % (self.robot.home, definition['driver']))

        if definition['platform'] == 'selenium':

            if definition['kind'] == 'driver':
                if definition['device'] == 'firefox':
                    binary = FirefoxBinary(definition['bin'])
                    profile = FirefoxProfile(profile_dir)
                    return FirefoxDriver(profile, binary)
                if definition['device'] == 'chrome':
                    service = Service(definition['driver'])
                    service.start()
                    capabilities = {'chrome.binary': definition['bin']}
                    return webdriver.Remote(service.service_url, capabilities)
                raise RallfError(-32001, "Selenium browser not supported, please use another factory")

            if definition['kind'] == 'remote':
                if definition['device'] == 'android':
                    capabilities = {
                        'automationName': 'uiautomator2',
                        'platformName': 'Android',
                        'deviceName': "test",
                        "appPackage": definition['app_package'],
                        "appActivity": definition['app_activity']
                    }
                    return appium_webdriver.Remote(command_executor="%s/wd/hub" % definition['server'], desired_capabilities=capabilities)

            raise RallfError(-32002, "Selenium kind not supported, please use another factory")
        raise RallfError(-32003, "Non-selenium devices are not supported, please use another factory")



import json
import pathlib

from rallf.sdk.robot import Robot


class RobotFactory:

    @staticmethod
    def createFromDir(dir):
        if not pathlib.Path(dir).is_dir():
            raise NotADirectoryError("%s is not a valid directory" % dir)
        return Robot(dir, json.load(open(dir + "/skills.json", "r")), json.load(open(dir + "/devices.json", "r")))

    def createEmpty(self):
        return Robot("/tmp/empty", {}, {})



import cv2


class ImageFinder:
    def __init__(self, pattern):
        self.pattern = cv2.imread(pattern, 0)

    def match(self, image):
        img = cv2.imread(image, 0)
        # Apply template Matching
        res = cv2.matchTemplate(img, self.pattern, cv2.TM_CCORR_NORMED)
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
        return {"matches": max_val, "coords": max_loc}



from rallf.tools.execution import Execution
import os


class Runner:
    def __init__(self):
        pass

    def execute(self, execution: Execution):
        DIR = execution.task.home

        if not os.path.exists(DIR): os.mkdir(DIR)
        os.chdir(DIR)
        # os.chroot(DIR)
        execution.task.warmup()
        execution.task.network.event("warmup:end")
        if execution.func is None: execution.task.waitloop()
        else: getattr(execution.task, execution.func)(execution.input)
        execution.task.cooldown()

        #os.unlink(DIR)

